//+------------------------------------------------------------------+
//|                                                     Blau_MDI.mq5 |
//|                        Copyright 2011, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2011, MetaQuotes Software Corp." //  -
#property link      "http://www.mql5.com"                       //    -
#property description "Mean Deviation Index (William Blau)"     //   mql5-
#include <WilliamBlau.mqh>              //   (   )
//---  
#property indicator_separate_window     //     
#property indicator_buffers 5           //     
#property indicator_plots   1           //     
//---   #0 (Main)
#property indicator_label1  "MDI"       //     #0
#property indicator_type1   DRAW_LINE   //  : DRAW_LINE - 
#property indicator_color1  Blue        //    : Blue - 
#property indicator_style1  STYLE_SOLID //  : STYLE_SOLID -  
#property indicator_width1  1           //  
//---  
input int    r=20; // r -  1- EMA,   
input int    s=5;  // s -  2- EMA,    
input int    u=3;  // u -  3- EMA,     
input ENUM_APPLIED_PRICE AppliedPrice=PRICE_CLOSE; // AppliedPrice -  
//---     
double MainBuffer[];      // u- 3- EMA (   #0)
double PriceBuffer[];     //  
double EMA_PriceBuffer[]; // r- 1- EMA ( )
double MDBuffer[];        //  
double DEMA_MDBuffer[];   // s- 2- EMA
//---  
int    begin1, begin2, begin3, begin4; //  ,     
int    rates_total_min; //     
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//---        
   //   ;     
   //   #0
   SetIndexBuffer(0,MainBuffer,INDICATOR_DATA);              // u- 3- EMA
   //     ;    
   SetIndexBuffer(1,PriceBuffer,INDICATOR_CALCULATIONS);     //  
   SetIndexBuffer(2,EMA_PriceBuffer,INDICATOR_CALCULATIONS); // r- 1- EMA ( )
   SetIndexBuffer(3,MDBuffer,INDICATOR_CALCULATIONS);        //  
   SetIndexBuffer(4,DEMA_MDBuffer,INDICATOR_CALCULATIONS);   // s- 2- EMA
/*
//---   #0 (Main)
   PlotIndexSetString(0,PLOT_LABEL,"MDI");             //     #0
   PlotIndexSetInteger(0,PLOT_DRAW_TYPE,DRAW_LINE);    //  : DRAW_LINE - 
   PlotIndexSetInteger(0,PLOT_LINE_COLOR,Blue);        //    : Blue - 
   PlotIndexSetInteger(0,PLOT_LINE_STYLE,STYLE_SOLID); //  : STYLE_SOLID -  
   PlotIndexSetInteger(0,PLOT_LINE_WIDTH,1);           //  
*/
//---    
   IndicatorSetInteger(INDICATOR_DIGITS,_Digits);
//---
   begin1=0;          //                         -  : 
   begin2=begin1+r-1; // or =0+(r-1)             -  : MDBuffer[], EMA_PriceBuffer[]
   begin3=begin2+s-1; // or =0+(r-1)+(s-1)       -  : DEMA_MDBuffer[]
   begin4=begin3+u-1; // or =0+(r-1)+(s-1)+(u-1) -  : MainBuffer[]
   //
   rates_total_min=begin4+1; //     
//---        #0
   PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,begin4);
//---   
   string shortname=PriceName(AppliedPrice)+","+string(r)+","+string(s)+","+string(u);
   IndicatorSetString(INDICATOR_SHORTNAME,"Blau_MDI("+shortname+")");
//--- OnInit done
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(
                const int rates_total,     //   
                const int prev_calculated, //     
                const datetime &Time[],    // Time
                const double &Open[],      // Open
                const double &High[],      // High
                const double &Low[],       // Low
                const double &Close[],     // Close
                const long &TickVolume[],  // Tick Volume
                const long &Volume[],      // Real Volume
                const int &Spread[]        // Spread
               )
  {
   int i,pos;
//---      
   if(rates_total<rates_total_min) return(0);
//---    PriceBuffer[]
   CalculatePriceBuffer(
                        AppliedPrice,        //  
                        rates_total,         //   
                        prev_calculated,     //     
                        Open,High,Low,Close, //  Open[], High[], Low[], Close[]
                        PriceBuffer          //   
                       );
//---   
   //   (pos),   /   
   //     MDBuffer[]
   if(prev_calculated==0)      //   
     {
      pos=begin2;              //  c  ,    
      for(i=0;i<pos;i++)       //   
         MDBuffer[i]=0.0;      //  
     }
   else pos=prev_calculated-1; //     
   // r- 1- EMA:   ( )   EMA_PriceBuffer[]
   ExponentialMAOnBufferWB(rates_total,prev_calculated,begin1,r,PriceBuffer,EMA_PriceBuffer);
   //     MDBuffer[]
   for(i=pos;i<rates_total;i++)
      MDBuffer[i]=PriceBuffer[i]-EMA_PriceBuffer[i];
//---   EMA
   // s- 2- EMA
   ExponentialMAOnBufferWB(
                           rates_total,     //   
                           prev_calculated, //     
                           begin2,          //         
                           s,               //  
                           MDBuffer,        //  
                           DEMA_MDBuffer    //  
                          );
   // u- 3- EMA (   #0)
   ExponentialMAOnBufferWB(rates_total,prev_calculated,begin3,u,DEMA_MDBuffer,MainBuffer);
//--- OnCalculate done. Return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+
